local name, addon = ...;

GamePad = {};

local gamepadInst;


function GamePad:new(o)
  o = o or {};   -- create object if user does not provide one
  setmetatable(o, self);
  self.__index = self;
  
  return o;
end

function getGamePad()
  if (gamepadInst == nil) then
    gamepadInst = GamePad:new();
  end
  return gamepadInst;
end

function GamePad:onAddonLoaded(frame, event, ...)
end

function GamePad:OnEvent(self, event, ...)
  local frame = self;
  local gamePad = getGamePad();
  if (event == "ADDON_LOADED") then
    gamePad:onAddonLoaded(frame, event, ...);
  end
end

function GamePad:OnLoad(frame)
  frame:RegisterEvent("ADDON_LOADED");
end

function GamePad:getFrame(text, arg1)
	if (not self.frame) then
			local backdrop = {
				bgFile = "Interface/BUTTONS/WHITE8X8",
				edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
				tile = true,
				edgeSize = 7,
				tileSize = 7,
				insets = {
					left = 2,
					right = 2,
					top = 2,
					bottom = 2,
				},
			}

		local f = CreateFrame("Frame", "GamePad", UIParent, BackdropTemplateMixin and "BackdropTemplate");
		self.frame = f;
		f:SetBackdrop(backdrop);
		f:SetBackdropColor(0, 0, 0);
		f:SetPoint("LEFT", UIParent, 20, 0);
		f:SetSize(700, 800);
		f:SetMovable(true);  -- The rest is pretty much to make the frame dragable
		f:EnableMouse(true);
		f:RegisterForDrag("LeftButton");
		f:SetScript("OnDragStart", f.StartMoving);
		f:SetScript("OnDragStop", f.StopMovingOrSizing);
		f:SetClampedToScreen(true);
		f.Close = CreateFrame("Button", "$parentClose", f, "UIPanelCloseButton");
		f.Close:SetPoint("TOPRIGHT", -2, -2);
		f.Close:SetScript("OnClick", function(self)
			self:GetParent():Hide();
			local gamePad = getGamePad();
			if (gamePad.timer ~= nil) then
			  gamePad.timer:Cancel();
			end
		end);
		f.Scroll = CreateFrame("ScrollFrame", "$parentScroll", f, "UIPanelScrollFrameTemplate");
		f.Scroll:SetPoint("TOPLEFT", f, 5, -30);
		f.Scroll:SetPoint("BOTTOMRIGHT", f, -30, 5);
		f.Text = CreateFrame("EditBox", "$parentEditbox", f);
		f.Text:SetMultiLine(true);
		f.Text:SetSize(f:GetSize());
		f.Text:SetPoint("TOPLEFT", f.Scroll);
		f.Text:SetPoint("BOTTOMRIGHT", f.Scroll);
		f.Text:SetMaxLetters(99999);
		f.Text:SetFontObject(GameFontNormal);
		f.Text:SetAutoFocus(false);
		f.Text:SetScript("OnEscapePressed", function(self) 
			self:ClearFocus();
		end);
		f.Scroll:SetScrollChild(f.Text);
	end
	if not self.frame:IsShown() then
		self.frame:Show();
	end	
	return self.frame;
end

function GamePad:Info()
  local deviceID = self.deviceID;
  local f = self:getFrame();
  local t = f.Text;
  local i;
  local msg;
  msg = string.format("Device %d\n", deviceID);
  --DEFAULT_CHAT_FRAME:AddMessage(msg);
  local rawState = C_GamePad.GetDeviceRawState(deviceID);
  local ypos = -100;
  if (rawState ~= nil) then
    msg = msg..string.format("Name: %s\nVendor: 0x%x\nProduct: 0x%x\nButtons: %d\nAxis: %d\n",
	   rawState.name 
	  , rawState.vendorID
	  , rawState.productID
	  , rawState.rawButtonCount
	  , rawState.rawAxisCount);
	  for i=1,rawState.rawButtonCount,1 do
	    local key = string.format("checkbuttonRaw%d", i);
	    local cb;
	    if (f[key] == nil) then
	      local tx = CreateFrame("EditBox", string.format("$parentEdit%d", i), f);
	      tx:SetFontObject(GameFontNormal);
		  tx:SetPoint("TOPLEFT", f, 20, ypos);
	      tx:SetWidth(100);
	      tx:SetHeight(24);
	      tx:SetAutoFocus(false);
	      tx:SetText(string.format("raw %d",i-1));
	      --tx:Disable();
	      cb = CreateFrame("CheckButton", string.format("$parentCheckboxRaw%d", i), f, "UICheckButtonTemplate");
		  cb:SetPoint("TOPLEFT", f, 65, ypos);
		  ypos = ypos - 20;
		  cb:SetDisabledFontObject(GameFontNormal);
		  cb:SetNormalFontObject(GameFontNormal);
		  --cb:SetWidth(100);
	      cb:Disable();
	      f[key] = cb;
	    else
	      cb = f[key];
	    end
	    cb:SetChecked(rawState.rawButtons[i]);
		--msg = msg..string.format(" %s", rawState.rawButtons[i] and "t" or "f");
	  end
	  ypos = ypos - 10;
	  for i=1,rawState.rawAxisCount,1 do
		local sli;
	    local key = string.format("axisRaw%d", i);
	    if (f[key] == nil) then
	      sli = CreateFrame("Slider", string.format("$parentSliderRaw%d", i), f, "OptionsSliderTemplate");
		  sli:SetPoint("TOPLEFT", f, 20, ypos);
		  ypos = ypos - 25;
		  sli:SetWidth(100);
	      sli:Disable();
	      _G[sli:GetName() .. 'Low']:SetText(string.format("raw %d",i-1)); 
 	      sli:SetOrientation('HORIZONTAL');
 	      sli:SetMinMaxValues(-1, 1);
	      f[key] = sli;
	    else
	      sli = f[key];
	    end
	    sli:SetValue(rawState.rawAxes[i]);
	  end  
  end    
  local state = C_GamePad.GetDeviceMappedState(deviceID);
  if (state ~= nil) then 
    local name = (state.name ~= nil) and state.name or "nil";
    local style = (state.style ~= nil) and state.style or "nil";
    --msg = msg..string.format("\n %s %s btn %d axi %d sti %d", name, style, state.buttonCount, state.axisCount, state.stickCount);
    local ypos = -100;
    -- indexes
    -- 1 forw., 2 right, 3 back, 4 left
    for i=1,state.buttonCount,1 do
  	    local conf = C_GamePad.ButtonIndexToConfigName(i-1);
	    local bind = C_GamePad.ButtonIndexToBinding(i-1);
   	    local key = string.format("CheckboxMap%d", i);
	    local cb;
	    if (f[key] == nil) then
	      local tx = CreateFrame("EditBox", string.format("$parentEditMap%d", i), f);
	      tx:SetFontObject(GameFontNormal);
		  tx:SetPoint("TOPLEFT", f, 160, ypos);
	      tx:SetWidth(400);
	      tx:SetHeight(24);
	      tx:SetAutoFocus(false);
	      tx:SetText(string.format("c:%s b:%s", conf ~= nil and conf or "", bind ~= nil and bind or ""));
	      cb = CreateFrame("CheckButton", string.format("$parentCheckboxMap%d", i), f, "UICheckButtonTemplate");
		  cb:SetPoint("TOPLEFT", f, 130, ypos);
		  ypos = ypos - 20;
		  cb:SetDisabledFontObject(GameFontNormal);
		  cb:SetNormalFontObject(GameFontNormal);
		  --cb:SetWidth(100);
	      cb:Disable();
	      f[key] = cb;
	    else
	      cb = f[key];
	    end
	    cb:SetChecked(state.buttons[i]);
    end
    local ypos = -100;
    for i=1,state.axisCount,1 do
   		local sli;
	    local key = string.format("axisMap%d", i);
	    if (f[key] == nil) then
	      sli = CreateFrame("Slider", string.format("$parentSliderMap%d", i), f, "OptionsSliderTemplate");
		  sli:SetPoint("TOPLEFT", f, 460, ypos);
		  ypos = ypos - 25;
		  sli:SetWidth(150);
	      sli:Disable();
  	      local conf = C_GamePad.AxisIndexToConfigName(i-1);
	      _G[sli:GetName() .. 'Low']:SetText(string.format("map %d %s",i-1, conf ~= nil and conf or "")); 
 	      sli:SetOrientation('HORIZONTAL');
 	      sli:SetMinMaxValues(-1, 1);
	      f[key] = sli;
	    else
	      sli = f[key];
	    end
	    sli:SetValue(state.axes[i]);
	  --msg = msg..string.format(" %d", state.axes[i]);
    end
    for i=1,state.stickCount,1 do
      local stick = state.sticks[i];
        -- this unifies the axis to a stick
        -- first to a "symbolic" name e.g. right, left...
        -- the to a in game target e.g. "Camera","Movement"...
        local tx;
	    local key = string.format("stickMap%d", i);
	    if (f[key] == nil) then
   	      tx = CreateFrame("EditBox", string.format("$parentAxisMap%d", i), f);
	      tx:SetFontObject(GameFontNormal);
		  tx:SetPoint("TOPLEFT", f, 460, ypos);
	      tx:SetWidth(250);
	      tx:SetHeight(24);
	      tx:SetAutoFocus(false);
	      ypos = ypos - 25;
          f[key] = tx;
	    else
	      tx = f[key];
	    end;	  
	    local conf = C_GamePad.StickIndexToConfigName(i-1);
        tx:SetText(string.format("Stick %s %.2f,%.2f,%.2f", conf ~= nil and conf or "", stick.x, stick.y, stick.len));
    end
  end 
  t:SetText(msg);
end

function GamePad:showButtonBind()
  local msg = "";
  local i = 0;
  while true do
  	local conf = C_GamePad.ButtonIndexToConfigName(i);
	local bind = C_GamePad.ButtonIndexToBinding(i);
	if not conf or not bind then 
	   break;
	end;
    msg = msg..string.format("\n conf %s bind %s ", conf, bind);
    i = i + 1;
  end
  return msg;
end

function GamePad:showStickBind()
  local msg = "";
  local i = 0;
  while true do
	local name = C_GamePad.StickIndexToConfigName(i);
	if not name then 
	   break;
	end;
	msg = msg..string.format("\n stick ", name);
	i = i + 1;
  end;
  return msg;
end;

function GamePad:showAxisBind()
  local msg = "";
  local i = 0;
  while true do
	local name = C_GamePad.AxisIndexToConfigName(i);
	if not name then 
	   break;
	end;
	msg = msg..string.format("\n axis ", name);
	i = i + 1;
  end;
  return msg;
end;

function GamePad:Slash(args, editbox)
  local enabled = C_GamePad.IsEnabled();
  local i;
  local msg;
  if (enabled) then
     --local deviceID = C_GamePad.GetCombinedDeviceID();	-- GetActiveDeviceID all report the highest device
     --msg = msg..string.format(" dev %d", deviceID);
     msg = string.format("enabled");
    -- msg = msg..GamePad:showButtonBind();
    --msg = msg .. GamePad:showStickBind();
    --msg = msg .. GamePad:showAxisBind();
    local configIDs = C_GamePad.GetAllConfigIDs();            
    for i=1,#configIDs,1 do
      local configID = configIDs[i]; 
      msg = msg..string.format("\n config vendor 0x%x produdct 0x%x", configID.vendorID, configID.productID);		 
    end
    local freeLook = IsGamePadFreelookEnabled();
    msg = msg..string.format("\n freelook %s",
	  freeLook and "t" or "f");
    --SetGamePadFreeLook(true);	-- not what i wanted ...     
    DEFAULT_CHAT_FRAME:AddMessage(msg);
    local deviceIDs = C_GamePad.GetAllDeviceIDs();
    if (#deviceIDs > 0) then
      local gamePad = getGamePad();
      gamePad.deviceID = deviceIDs[1];
      if (gamePad.timer == nil or gamePad.timer:IsCancelled()) then 
	    gamePad.timer = C_Timer.NewTicker(0.5, function()
	      local gamePad = getGamePad();
	      gamePad:Info();
	    end);
	  end;
	end;
    for i=1,#deviceIDs,1 do
      msg = string.format("devs %d", deviceIDs[i]);
        local state = C_GamePad.GetDeviceMappedState(deviceID);
      if (state ~= nil) then 
        local name = (state.name ~= nil) and state.name or "nil";
        local style = (state.style ~= nil) and state.style or "nil";
        msg = msg..string.format(" name %s styl %s", name, style);
      end
      DEFAULT_CHAT_FRAME:AddMessage(msg);
    end
    --if (deviceID > 0) then
    --    local configID = {};
    --    configID.vendorId = 0x04d6;
    --    configID.productId = 0xc20b;
    --    local config = C_GamePad.GetConfig(configID);
    --    msg = msg..string.format("\n conf name %s",
    --       (config ~= nil) and config.name or "nil");
    --end
   else 
     msg = string.format("disabled");
    DEFAULT_CHAT_FRAME:AddMessage(msg);
   end
end
		
SLASH_GAMEPAD1 = "/gamepad"
SlashCmdList["GAMEPAD"] = function(args, editbox)
	GamePad:Slash(args, editbox)
end